--[[ 

	Enhancement Shaman PVE Helper
	Martin Karer / Sezz, 2010

	Changelog:
	2010-01-20:
		[+] Fire Totem Check
		[+] Fire Nova Check

	2010-01-19:
		[*] Removed nearly all textures
		[+] Flame Shock Debuff Check
		
	2010-01-18:
		[+] Re-Design
		[+] Maelstrom Weapon Tick counter
		[+] Spell Icons
		[+] Weapon Imbue Checks 
		
	2010-01-17:
		[~] Testing: Maelstrom Weapon tick counter

--]]

local addon = CreateFrame("Frame", "sEnhancer")
local _G = getfenv(0)
local LWE = LibWeaponEnchant;

local buttonSize = 24
local buttonBorderSize = 2
local buttonPadding = 8

local FADE_TIME = 0.2;
local ALWAYS_VISIBLE = true;
local BUFF_WARN_TIME = 7; -- FS only ticks every 3 seconds!

-- Colors/Alpha Values
local tickGlossAlpha = {
	[0] = 0.25,
	[1] = 0.50,
	[2] = 0.50,
	}

local tickColors = {
	[0] = { ["r"] = 0, ["g"] = 0, ["b"] = 0, ["a"] = 1 }, -- 0 ticks
	[1] = { ["r"] = 0/255, ["g"] = 158/255, ["b"] = 197/255, ["a"] = 1 }, -- 1-4 ticks
	[2] = { ["r"] = 204/255, ["g"] = 0/255, ["b"] = 255/255, ["a"] = 1 }, -- 5 ticks
	}

local tickBorderColors = {
	[0] = { ["r"] = .6, ["g"] = .6, ["b"] = .6, ["a"] = 1 },
	[1] = { ["r"] = 1, ["g"] = 1, ["b"] = 1, ["a"] = 1 },
	[2] = { ["r"] = 1, ["g"] = 1, ["b"] = 1, ["a"] = 1 },
	}

local spellColors = {
	[0] = { ["r"] = 1, ["g"] = 1, ["b"] = 1, ["a"] = .2 }, -- fadet out
	[1] = { ["r"] = 0.3, ["g"] = 0.5, ["b"] = 1, ["a"] = 1 }, -- needs attention soon
	[2] = { ["r"] = 1, ["g"] = 1, ["b"] = 1, ["a"] = 1 }, -- warning
	}

local spellBorderColors = {
	[0] = { ["r"] = 1, ["g"] = 1, ["b"] = 1, ["a"] = 0.5 },
	[1] = { ["r"] = 1, ["g"] = 0.5, ["b"] = 0, ["a"] = 1.0 },
	[2] = { ["r"] = 1, ["g"] = 0, ["b"] = 0, ["a"] = 1.0 },
	}

-- Spell Data
addon.SpellButtonsOrder = { "Flame Shock", "Magma Totem", "Fire Nova", "Windfury Weapon", "Flametongue Weapon" }
addon.SpellData = {
	["Flame Shock"] = { GetSpellInfo(49233) },
	["Magma Totem"] = { GetSpellInfo(58734) },
	["Fire Nova"] = { GetSpellInfo(1535) },
	["Windfury Weapon"] = { GetSpellInfo(58804) },
	["Flametongue Weapon"] = { GetSpellInfo(58790) },
	}

addon.SpellButtonsIndex = {}
for i = 1, 5 do
	addon.SpellButtonsIndex[addon.SpellButtonsOrder[i]] = i;
end

-- Events
local _eventsCombat = {
	"UNIT_AURA", -- Maelstrom Weapon, Flame Shock
	"PLAYER_TARGET_CHANGED", -- Flame Shock
	"COMBAT_LOG_EVENT_UNFILTERED", -- Flame Shock
	"PLAYER_TOTEM_UPDATE", -- Fire Totem, Fire Nova
	"SPELL_UPDATE_COOLDOWN", -- Fire Nova
	}

addon:RegisterEvent("PLAYER_LOGIN");
addon:RegisterEvent("PLAYER_REGEN_ENABLED");
addon:RegisterEvent("PLAYER_REGEN_DISABLED");
--addon:RegisterEvent("UNIT_AURA");
--addon:RegisterEvent("PLAYER_TARGET_CHANGED");
--addon:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED");
--addon:RegisterEvent("PLAYER_TOTEM_UPDATE");
--addon:RegisterEvent("SPELL_UPDATE_COOLDOWN");

-- (De)Buff Checking
local UnitHasBuff = function(unit, buffName, debuff) -- returns expirationTime, index, count, unitCaster
	-- name, rank, icon, count, debuffType, duration, expirationTime, unitCaster, isStealable, shouldConsolidate, spellId
	local i = 1;
	local name, rank, icon, count, debuffType, duration, expirationTime, unitCaster, isStealable, shouldConsolidate, spellId;
	while true do
		if (debuff) then
			name, rank, icon, count, debuffType, duration, expirationTime, unitCaster, isStealable, shouldConsolidate, spellId = UnitDebuff(unit, i);
		else
			name, rank, icon, count, debuffType, duration, expirationTime, unitCaster, isStealable, shouldConsolidate, spellId = UnitAura(unit, i);
		end
		
		if (name) then
			if (name == buffName) then
				return expirationTime, i, count, unitCaster;
			end
			i = i + 1;
		else
			return nil, 0, 0, 0;
		end
	end
end

-- Flame Shock
function addon:CheckFlameShockDebuff()
	local debuffState = 0
	if (UnitExists("target") and UnitCanAttack("player", "target") and not UnitIsDead("target")) then
		local expirationTime, hasDebuff, _, unitCaster = UnitHasBuff("target", "Flame Shock", true)
		if (hasDebuff ~= nil and unitCaster == "player") then
			debuffState = expirationTime - GetTime() < BUFF_WARN_TIME and 1 or 0;
		else
			debuffState = 2;
		end
	end

	addon:ToggleSpellStatus(addon.SpellButtonsIndex["Flame Shock"], debuffState);
end

-- Fire Totem, Fire Nova
function addon:CheckFireTotem()
	if (UnitAffectingCombat("player")) then
		local haveTotem, totemName, startTime, duration = GetTotemInfo(FIRE_TOTEM_SLOT);

		addon:ToggleSpellStatus(addon.SpellButtonsIndex["Magma Totem"], totemName ~= "" and 0 or 2);
		addon:ToggleSpellStatus(addon.SpellButtonsIndex["Fire Nova"], totemName == "" and 0 or (IsUsableSpell("Fire Nova") and 2 or 0));
	else
		addon:ToggleSpellStatus(addon.SpellButtonsIndex["Magma Totem"], 0);
		addon:ToggleSpellStatus(addon.SpellButtonsIndex["Fire Nova"], 0);
	end
end

local GetMaelstromWeaponTicks = function()
	local _, _, ticks = UnitHasBuff("player", "Maelstrom Weapon");
	return ticks;
end

-- Color Fading
local getFadeStep = function(oldNum, newNum)
	local numDiff = oldNum - newNum;
	return (numDiff ~= 0 and numDiff / 100 or 0);
end

local calculcateFadeColor = function(oldColors, fadeColors, fadePercent) 
	return oldColors["r"] - (fadePercent * getFadeStep(oldColors["r"], fadeColors["r"])),
		   oldColors["g"] - (fadePercent * getFadeStep(oldColors["g"], fadeColors["g"])),
		   oldColors["b"] - (fadePercent * getFadeStep(oldColors["b"], fadeColors["b"])),
		   oldColors["a"] - (fadePercent * getFadeStep(oldColors["a"], fadeColors["a"]));
end

local tickButton_OnUpdate = function(tickButton)
	if (tickButton.fadeInStartTime ~= nil and tickButton.tickStateOld ~= tickButton.tickState) then
		-- Fade...
		local fadePercent = ((GetTime() - tickButton.fadeInStartTime) / FADE_TIME) * 100;
		if (fadePercent > 100) then fadePercent = 100; end;
		
		if (tickButton.colorTable == "TICK") then
			tickButton.Icon:SetVertexColor(calculcateFadeColor(tickColors[tickButton.tickStateOld], tickColors[tickButton.tickState], fadePercent));
			tickButton.Border:SetVertexColor(calculcateFadeColor(tickBorderColors[tickButton.tickStateOld], tickBorderColors[tickButton.tickState], fadePercent));
		elseif (tickButton.colorTable == "SPELL") then
			tickButton.Icon:SetVertexColor(calculcateFadeColor(spellColors[tickButton.tickStateOld], spellColors[tickButton.tickState], fadePercent));
			tickButton.Border:SetVertexColor(calculcateFadeColor(spellBorderColors[tickButton.tickStateOld], spellBorderColors[tickButton.tickState], fadePercent));
		end
		
		tickButton.Gloss:SetAlpha(tickGlossAlpha[tickButton.tickStateOld] - (fadePercent * getFadeStep(tickGlossAlpha[tickButton.tickStateOld], tickGlossAlpha[tickButton.tickState])));
		
		if (fadePercent == 100) then
			tickButton.fadeInStartTime = nil;
			tickButton:SetScript("OnUpdate", nil);
		end
	else
		tickButton.fadeInStartTime = nil;
		tickButton:SetScript("OnUpdate", nil);
	end
end

function addon:SetMaelstromTicks(tickNum)
	for i = 1, 5 do
		local tickButton = addon.MaelstromTicks[i];
		if (tickButton) then
			local tickState	= ((tickNum == 5) and 2 or ((i <= tickNum) and 1 or 0));
			if (tickState ~= tickButton.tickState) then
				tickButton.tickStateOld = tickButton.tickState;
				tickButton.tickState = tickState;
				tickButton.fadeInStartTime = GetTime();
				tickButton:SetScript("OnUpdate", tickButton_OnUpdate);
			end
		else
			return;
		end
	end
end

-- IC/OOC Visibility
function addon:ToggleVisibility(showAddon)
	if (showAddon and addon:GetAlpha() ~= 1) then
		UIFrameFadeIn(sEnhancer, 0.2);
	elseif (not showAddon and addon:GetAlpha() ~= 0) then
		UIFrameFadeOut(sEnhancer, 0.2);
	end
end

-- Button Template
-- TODO: XML
function addon:CreateButton(buttonName, borderColor, iconTexture, iconColor)
	local spellButton = CreateFrame("Button", buttonName, addon);
	spellButton:SetWidth(buttonSize);
	spellButton:SetHeight(buttonSize);
	spellButton:SetParent(addon);
	spellButton:EnableMouse(false);
	spellButton.tickState = 0;

	-- Border
	local spellButtonBorder = spellButton:CreateTexture(buttonName.."Border", "BACKGROUND");
	spellButtonBorder:SetBlendMode("BLEND");
	spellButtonBorder:SetAllPoints(spellButton);
	spellButtonBorder:SetTexture([[Interface\Tooltips\UI-Tooltip-Background]]);
	spellButtonBorder:SetTexCoord(0.08, 0.92, 0.08, 0.92);
	spellButtonBorder:SetVertexColor(1, 1, 1, 0.5);
	spellButton.Border = spellButtonBorder

	-- Black Icon BG
	local spellButtonIconBackground = spellButton:CreateTexture(buttonName.."IconBackground", "BORDER");
	spellButtonIconBackground:SetBlendMode("MOD");
	spellButtonIconBackground:ClearAllPoints();
	spellButtonIconBackground:SetPoint("CENTER");
	spellButtonIconBackground:SetWidth(buttonSize - 2 * buttonBorderSize);
	spellButtonIconBackground:SetHeight(buttonSize - 2 * buttonBorderSize);
	spellButtonIconBackground:SetTexture([[Interface\Addons\sEnhancer\media\overlay]]);
	spellButtonIconBackground:SetTexCoord(0.08, 0.92, 0.08, 0.92);
	spellButtonIconBackground:SetVertexColor(0, 0, 0, 1);

	-- Icon
	local spellButtonIcon = spellButton:CreateTexture(buttonName.."Icon", "ARTWORK");
	spellButtonIcon:ClearAllPoints();
	spellButtonIcon:SetPoint("CENTER");
	spellButtonIcon:SetWidth(buttonSize - 2 * buttonBorderSize);
	spellButtonIcon:SetHeight(buttonSize - 2 * buttonBorderSize);
	spellButtonIcon:SetTexture(iconTexture);
	spellButtonIcon:SetTexCoord(0.08, 0.92, 0.08, 0.92);
	spellButtonIcon:SetVertexColor(iconColor["r"], iconColor["g"], iconColor["b"], iconColor["a"]);
	spellButton.Icon = spellButtonIcon

	-- Gloss
	local spellButtonGloss = spellButton:CreateTexture(buttonName.."Gloss", "OVERLAY");
	spellButtonGloss:SetBlendMode("ADD");
	spellButtonGloss:ClearAllPoints();
	spellButtonGloss:SetPoint("CENTER");
	spellButtonGloss:SetWidth(buttonSize - 2 * buttonBorderSize);
	spellButtonGloss:SetHeight(buttonSize - 2 * buttonBorderSize);
	spellButtonGloss:SetTexture([[Interface\Addons\sEnhancer\media\gloss]]);
	spellButtonGloss:SetVertexColor(1,1,1,1);
	spellButtonGloss:SetAlpha(0.25);
	spellButton.Gloss = spellButtonGloss

	return spellButton;
end

function addon:ToggleSpellStatus(spellNum, newState)
	local spellButton = addon.SpellButtons[spellNum];
	if (spellButton and spellButton.tickState ~= newState) then
		spellButton.tickStateOld = spellButton.tickState;
		spellButton.tickState = newState;
		spellButton.fadeInStartTime = GetTime();
		spellButton:SetScript("OnUpdate", tickButton_OnUpdate);
	end
end

-- Weapon Enchants
-- Using LibWeaponEnchant by Aezay
-- http://www.wowinterface.com/downloads/info8437.html
addon.UpdateWeaponEnchants = function(mhSlot,mhEnchant,mhDuration,mhEndTime,ohSlot,ohEnchant,ohDuration,ohEndTime)
	addon:CheckWeaponEnchants();
end

function addon:CheckWeaponEnchants()
	local mhSlot, mhEnchant, mhDuration, mhEndTime, ohSlot, ohEnchant, ohDuration, ohEndTime = LWE.GetEnchantData();
	local mhEnchantName = mhEnchant and mhEnchant:gsub(" %d+","") or "UNKNOWN";
	local ohEnchantName = ohEnchant and ohEnchant:gsub(" %d+","") or "UNKNOWN";

	addon:ToggleSpellStatus(addon.SpellButtonsIndex["Windfury Weapon"], mhEnchantName == "Windfury" and 0 or 2);
	addon:ToggleSpellStatus(addon.SpellButtonsIndex["Flametongue Weapon"], ohEnchantName == "Flametongue" and 0 or 2);
end

LWE.RegisterCallback(addon, addon.UpdateWeaponEnchants);

-- Event Handlers
addon:SetScript("OnEvent", function(self, event ,...)
	if (event == "PLAYER_LOGIN") then
		local _, englishClass = UnitClass("player");
		if (englishClass ~= "SHAMAN") then
			LWE.UnregisterCallback(addon);
			addon:UnregisterAllEvents();
			return;
		end

		addon:SetParent(UIParent);
		addon:ClearAllPoints();
		addon:SetPoint("CENTER", 0, -130);
		addon:SetWidth(buttonSize * 5 + buttonPadding * 4);
		addon:SetHeight(buttonSize * 2 + buttonPadding);
		addon:SetFrameStrata("LOW");
		
		if (not ALWAYS_VISIBLE) then
			addon:SetAlpha(0);
		end

		-- Create Maelstrom Tick Frames
		addon.MaelstromTicks = {};
		for i = 1, 5 do
			local tickButton = addon:CreateButton(addon:GetName().."MaelstromTick"..i, tickBorderColors[0], [[Interface\Addons\sEnhancer\media\overlay]], tickColors[0]);
			tickButton:ClearAllPoints();
			tickButton.colorTable = "TICK";
			if (i == 1) then
				tickButton:SetPoint("TOPLEFT");
			else
				tickButton:SetPoint("LEFT", addon.MaelstromTicks[i - 1], "RIGHT", buttonPadding, 0);
			end
			addon.MaelstromTicks[i] = tickButton;
		end
		
		-- Init Maelstrom Tick Count
		addon:SetMaelstromTicks(0);
		
		-- Create (De)Buff Frames
		addon.SpellButtons = {};
		for i = 1, 5 do
			local spellButton = addon:CreateButton(addon:GetName().."SpellButton"..i, spellBorderColors[0], addon.SpellData[addon.SpellButtonsOrder[i]][3], spellColors[0]);
			spellButton:ClearAllPoints();
			spellButton.colorTable = "SPELL";
			if (i == 1) then
				spellButton:SetPoint("TOPLEFT", addon.MaelstromTicks[1], "BOTTOMLEFT", 0, buttonPadding * -1);
			else
				spellButton:SetPoint("LEFT", addon.SpellButtons[i - 1], "RIGHT", buttonPadding, 0);
			end
			addon.SpellButtons[i] = spellButton;
		end

		-- Check Weapon Enchants
		addon:CheckWeaponEnchants();
		
		-- Check Fire Totem
		addon:CheckFireTotem();
		
		-- Done
		addon:UnregisterEvent("PLAYER_LOGIN");
	elseif (event == "PLAYER_REGEN_ENABLED") then
		-- Out Of Combat: DISABLE
		if (not ALWAYS_VISIBLE) then
			-- Hide Status Frames
			addon:ToggleVisibility(false);
		end

		-- Unregister Events
		for _, v in pairs(_eventsCombat) do
			addon:UnregisterEvent(v);
		end

		-- Fade-Out Fire Totem
		addon:CheckFireTotem();
	elseif (event == "PLAYER_REGEN_DISABLED") then
		-- In Combat: ENABLE
		if (not ALWAYS_VISIBLE) then
			-- Display Status Frames
			addon:ToggleVisibility(true);
		end

		-- Register Events
		for _, v in pairs(_eventsCombat) do
			addon:RegisterEvent(v);
		end

		-- Fire Totem Check
		addon:CheckFireTotem();
	elseif (event == "UNIT_AURA") then
		if (arg1 ~= nil) then
			if (arg1 == "player") then
				-- Update Maelstrom Ticks
				addon:SetMaelstromTicks(GetMaelstromWeaponTicks());
			elseif (arg1 == "target") then
				-- Check Flame Shock Debuff
				addon:CheckFlameShockDebuff();
			end
		end
	elseif (event == "PLAYER_TARGET_CHANGED") then
		-- Check Flame Shock Debuff
		addon:CheckFlameShockDebuff();
	elseif (event == "COMBAT_LOG_EVENT_UNFILTERED") then
		-- Flame Shock Periodic Damage
		if (arg3 == UnitGUID("player") and arg6 == UnitGUID("target") and arg2 == "SPELL_PERIODIC_DAMAGE") then
			addon:CheckFlameShockDebuff();
		end
	elseif (event == "PLAYER_TOTEM_UPDATE") then
		if (arg1 == FIRE_TOTEM_SLOT) then
			addon:CheckFireTotem();
		end
	elseif (event == "SPELL_UPDATE_COOLDOWN") then
		addon:CheckFireTotem();
	end
end)

-- Done
_G["sEnhancer"] = addon

