--[[ 

	Enhancement Shaman PVE Helper
	Martin Karer / Sezz, 2010

	Changelog:
	2010-01-18:
		[+] Re-Design
		[+] Maelstrom Weapon tick counter should be fully working now :)
		[+] Spell Icons
		
	2010-01-17:
		[~] Testing: Maelstrom Weapon tick counter

--]]

local addon = CreateFrame("Frame", "sEnhancer")
local _G = getfenv(0)
local LWE = LibWeaponEnchant;

local buttonSize = 32
local buttonPadding = 8

local FADE_TIME = 0.2;
local ALWAYS_VISIBLE = true;
local BUFF_WARN_TIME = 5;

local tickGlossAlpha = {
	[0] = 0.25,
	[1] = 0.50,
	[2] = 0.50,
	}

local tickColors = {
	[0] = { ["r"] = 0, ["g"] = 0, ["b"] = 0, ["a"] = 1 },
	[1] = { ["r"] = 0/255, ["g"] = 158/255, ["b"] = 197/255, ["a"] = 1 },
	[2] = { ["r"] = 204/255, ["g"] = 0/255, ["b"] = 255/255, ["a"] = 1 },
	}

local tickBorderColors = {
	[0] = { ["r"] = .6, ["g"] = .6, ["b"] = .6, ["a"] = 1 },
	[1] = { ["r"] = 1, ["g"] = 1, ["b"] = 1, ["a"] = 1 },
	[2] = { ["r"] = 1, ["g"] = 1, ["b"] = 1, ["a"] = 1 },
	}

local spellColors = {
	[0] = { ["r"] = 1, ["g"] = 1, ["b"] = 1, ["a"] = .2 },
	[1] = { ["r"] = 1, ["g"] = 1, ["b"] = 1, ["a"] = 1 },
	}

local spellBorderColors = {
	[0] = { ["r"] = .6, ["g"] = .6, ["b"] = .6, ["a"] = 1 },
	[1] = { ["r"] = 1, ["g"] = 0, ["b"] = 0, ["a"] = 1 },
	}

--[[

"PLAYER_TOTEM_UPDATE"
	Category: Player
  	

This event fires whenever a totem is dropped (cast) or destroyed (either recalled or killed).

arg1 
    The number of the totem slot (1-4) affected by the update. See for example GetTotemInfo() for the uses of the totem slot number. 



"PLAYER_REGEN_ENABLED"
	Category: Player,Combat
  	

Fired after ending combat, as regen rates return to normal. Useful for determining when a player has left combat. This occurs when you are not on the hate list of any NPC, or a few seconds after the latest pvp attack that you were involved with. 

"PLAYER_REGEN_DISABLED"
	Category: Player,Combat
  	

Fired whenever you enter combat, as normal regen rates are disabled during combat. This means that either you are in the hate list of a NPC or that you've been taking part in a pvp action (either as attacker or victim). 

"UNIT_AURA"
	Category: Unit Info,Buff
  	

Fired when a buff, debuff, status, or item bonus was gained by or faded from a player, pet, NPC, or mob.

arg1 
    the UnitID of the entity 

1-28-06: This event IS NOT fired when changing targets any longer. 
    
--]]

-- Spells
addon.spells = {}
-- Weapon Imbue Warnings
addon.spells["Windfury Weapon"], _, addon.spells["Windfury Weapon Icon"] = GetSpellInfo(58804)
addon.spells["Flametongue Weapon"], _, addon.spells["Flametongue Weapon Icon"] = GetSpellInfo(58790)
-- Shocks
-- Show FS when FS is not on Target or Uptime <= 5s, otherwise show ES
addon.spells["Flame Shock"], _, addon.spells["Flame Shock Icon"] = GetSpellInfo(49233) -- Show when not on (Focus?) Target
addon.spells["Earth Shock"], _, addon.spells["Earth Shock Icon"] = GetSpellInfo(49231) -- Show when FS on Target
-- Maelstrom Tick Counter
-- Big indicator when counter reaches 5
addon.spells["Maelstrom Weapon"], _, addon.spells["Maelstrom Weapon Icon"] = GetSpellInfo(51532)
-- Fire Totems
-- Show current totem when uptime (nearly) 0
-- Show FN Cooldown
addon.spells["Fire Nova"], _, addon.spells["Fire Nova Icon"] = GetSpellInfo(1535)
addon.spells["Searing Totem"], _, addon.spells["Searing Totem Icon"] = GetSpellInfo(58704)
addon.spells["Magma Totem"], _, addon.spells["Magma Totem Icon"] = GetSpellInfo(58734)
addon.spells["Fire Elemental Totem"], _, addon.spells["Fire Elemental Totem Icon"] = GetSpellInfo(32982)

addon.SpellButtonsOrder = { "Flame Shock", "Magma Totem", "Fire Nova", "Windfury Weapon", "Flametongue Weapon" }
addon.SpellData = {
	["Flame Shock"] = { GetSpellInfo(49233) },
	["Magma Totem"] = { GetSpellInfo(58734) },
	["Fire Nova"] = { GetSpellInfo(1535) },
	["Windfury Weapon"] = { GetSpellInfo(58804) },
	["Flametongue Weapon"] = { GetSpellInfo(58790) },
	}

addon.SpellButtonsIndex = {}
for i = 1, 5 do
	addon.SpellButtonsIndex[addon.SpellButtonsOrder[i]] = i;
end

-- Buff Checking
local UnitHasBuff = function(unit, buffName) -- returns expirationTime, index, count
	-- name, rank, icon, count, debuffType, duration, expirationTime, unitCaster, isStealable, shouldConsolidate, spellId
	local i = 1;
	local name, rank, icon, count, debuffType, duration, expirationTime, unitCaster, isStealable, shouldConsolidate, spellId;
	while true do
		name, rank, icon, count, debuffType, duration, expirationTime, unitCaster, isStealable, shouldConsolidate, spellId = UnitAura(unit, i);
		if (name) then
			if (name == buffName) then
				return expirationTime, i ,count, tt;
			end
			i = i + 1;
		else
			return nil, i , 0, 0;
		end
	end
end

local GetMaelstromWeaponTicks = function()
	local _, _, ticks = UnitHasBuff("player", "Maelstrom Weapon");
	return ticks;
end

addon:RegisterEvent("UNIT_AURA");
addon:RegisterEvent("PLAYER_LOGIN");
addon:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED");
addon:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED");
if (not ALWAYS_VISIBLE) then
	addon:RegisterEvent("PLAYER_REGEN_ENABLED");
	addon:RegisterEvent("PLAYER_REGEN_DISABLED");
end

-- Color Fading
local getFadeStep = function(oldNum, newNum)
	local numDiff = oldNum - newNum;
	return (numDiff ~= 0 and numDiff / 100 or 0);
end

local calculcateFadeColor = function(oldColors, fadeColors, fadePercent) 
	return oldColors["r"] - (fadePercent * getFadeStep(oldColors["r"], fadeColors["r"])),
		   oldColors["g"] - (fadePercent * getFadeStep(oldColors["g"], fadeColors["g"])),
		   oldColors["b"] - (fadePercent * getFadeStep(oldColors["b"], fadeColors["b"])),
		   oldColors["a"] - (fadePercent * getFadeStep(oldColors["a"], fadeColors["a"]));
end

local tickButton_OnUpdate = function(tickButton)
	if (tickButton.fadeInStartTime ~= nil and tickButton.tickStateOld ~= tickButton.tickState) then
		-- Fade...
		local fadePercent = ((GetTime() - tickButton.fadeInStartTime) / FADE_TIME) * 100;
		if (fadePercent > 100) then fadePercent = 100; end;
		
		if (tickButton.colorTable == "TICK") then
			tickButton.Icon:SetVertexColor(calculcateFadeColor(tickColors[tickButton.tickStateOld], tickColors[tickButton.tickState], fadePercent));
			tickButton.Border:SetVertexColor(calculcateFadeColor(tickBorderColors[tickButton.tickStateOld], tickBorderColors[tickButton.tickState], fadePercent));
		elseif (tickButton.colorTable == "SPELL") then
			tickButton.Icon:SetVertexColor(calculcateFadeColor(spellColors[tickButton.tickStateOld], spellColors[tickButton.tickState], fadePercent));
			tickButton.Border:SetVertexColor(calculcateFadeColor(spellBorderColors[tickButton.tickStateOld], spellBorderColors[tickButton.tickState], fadePercent));
		end
		
		tickButton.Gloss:SetAlpha(tickGlossAlpha[tickButton.tickStateOld] - (fadePercent * getFadeStep(tickGlossAlpha[tickButton.tickStateOld], tickGlossAlpha[tickButton.tickState])));
		
		if (fadePercent == 100) then
			tickButton.fadeInStartTime = nil;
			tickButton:SetScript("OnUpdate", nil);
		end
	else
		tickButton.fadeInStartTime = nil;
		tickButton:SetScript("OnUpdate", nil);
	end
end

function addon:SetMaelstromTicks(tickNum)
	for i = 1, 5 do
		local tickButton = addon.MaelstromTicks[i];
		if (tickButton) then
			local tickState	= ((tickNum == 5) and 2 or ((i <= tickNum) and 1 or 0));
			if (tickState ~= tickButton.tickState) then
				tickButton.tickStateOld = tickButton.tickState;
				tickButton.tickState = tickState;
				tickButton.fadeInStartTime = GetTime();
				tickButton:SetScript("OnUpdate", tickButton_OnUpdate);
			end
		else
			return;
		end
	end
end

-- IC/OOC Visibility
function addon:ToggleVisibility(showAddon)
	if (showAddon and addon:GetAlpha() ~= 1) then
		UIFrameFadeIn(sEnhancer, 0.2);
	elseif (not showAddon and addon:GetAlpha() ~= 0) then
		UIFrameFadeOut(sEnhancer, 0.2);
	end
end

-- Button Template
-- TODO: XML
function addon:CreateButton(buttonName, borderColor, iconTexture, iconColor)
	local tickButton = CreateFrame("Button", buttonName, addon);
	tickButton:SetWidth(buttonSize);
	tickButton:SetHeight(buttonSize);
	tickButton:SetParent(addon);
	tickButton:SetNormalTexture([[Interface\Addons\sEnhancer\media\button]]);
	tickButton:EnableMouse(false);
	tickButton.tickState = 0;
	tickButton.isFading = false;
	--tick:SetVertexColor(1,0,0,1);

	-- Textures
	local tickBorder = tickButton:CreateTexture(buttonName.."Border", "OVERLAY");
	tickBorder:SetBlendMode("MOD");
	tickBorder:SetAllPoints(tickButton);
	tickBorder:SetTexture([[Interface\Addons\sEnhancer\media\border]]);
--	tickBorder:SetVertexColor(1/255, 206/255, 204/255, 1);
	tickBorder:SetVertexColor(borderColor["r"], borderColor["g"], borderColor["b"], borderColor["a"]);
	tickButton.Border = tickBorder

	local tickGloss = tickButton:CreateTexture(buttonName.."Gloss", "OVERLAY");
	tickGloss:SetBlendMode("ADD");
	tickGloss:SetAllPoints(tickButton);
	tickGloss:SetTexture([[Interface\Addons\sEnhancer\media\gloss]]);
	tickGloss:SetVertexColor(1,1,1,1);
	tickGloss:SetAlpha(0.25);
	tickButton.Gloss = tickGloss

	local tickIconBackground = tickButton:CreateTexture(buttonName.."IconBackground", "BACKGROUND");
	tickIconBackground:ClearAllPoints();
	tickIconBackground:SetPoint("CENTER");
	tickIconBackground:SetWidth(buttonSize - 8);
	tickIconBackground:SetHeight(buttonSize - 8);
	tickIconBackground:SetTexture([[Interface\Addons\sEnhancer\media\overlay]]);
	tickIconBackground:SetTexCoord(0.08, 0.92, 0.08, 0.92);
	tickIconBackground:SetVertexColor(0, 0, 0, 1);

	local tickIcon = tickButton:CreateTexture(buttonName.."Icon", "ARTWORK");
	tickIcon:ClearAllPoints();
	tickIcon:SetPoint("CENTER");
	tickIcon:SetWidth(buttonSize - 8);
	tickIcon:SetHeight(buttonSize - 8);
	tickIcon:SetTexture(iconTexture);
	tickIcon:SetTexCoord(0.08, 0.92, 0.08, 0.92);
	tickIcon:SetVertexColor(iconColor["r"], iconColor["g"], iconColor["b"], iconColor["a"]);
	tickButton.Icon = tickIcon

	return tickButton;
end

function addon:NEEDWF(really)
	local spellButton = addon.SpellButtons[4];
	if (spellButton and spellButton.tickState ~= really) then
		spellButton.tickStateOld = spellButton.tickState;
		spellButton.tickState = really;
		spellButton.fadeInStartTime = GetTime();
		spellButton:SetScript("OnUpdate", tickButton_OnUpdate);
	end
end

function addon:ToggleSpellStatus(spellNum)
	-- OK = DIM
	-- NOT OK = SHOW
end


-- Weapon Enchants
-- Using LibWeaponEnchant by Aezay
-- http://www.wowinterface.com/downloads/info8437.html
local enchantShort = { Flametongue = "FT", Windfury = "WF", Earthliving = "EL", Frostbrand = "FB", Rockbiter = "RB", [UNKNOWN] = "??" };
addon.UpdateWeaponEnchants = function(mhSlot,mhEnchant,mhDuration,mhEndTime,ohSlot,ohEnchant,ohDuration,ohEndTime)
	addon:ScanPlayerAuras();
end
function addon:ScanPlayerAuras()
	local mhSlot, mhEnchant, mhDuration, mhEndTime, ohSlot, ohEnchant, ohDuration, ohEndTime = LWE.GetEnchantData();

--	if (not mhEnchant) and (not ohEnchant) then
--		tbl.label = "No Weapon Imbue";
--	elseif (mhEnchant) and (not OffhandHasWeapon()) then
--		local mhShort = enchantShort[mhEnchant:gsub(" %d+","")] or mhEnchant;
--		sCore:Print("Weapon Imbue ("..mhShort..")";
--		tbl.icon = icon;
--		tbl.duration = mhDuration;
--		tbl.endTime = mhEndTime;
--		tbl.color = self.cfg.colGood;
--	else
		local mhShort = mhEnchant and enchantShort[mhEnchant:gsub(" %d+","")] or mhEnchant or "??";
		local ohShort = ohEnchant and enchantShort[ohEnchant:gsub(" %d+","")] or ohEnchant or "??";
		
		sCore:Print("Weapon Imbues ("..mhShort.."/"..ohShort..")");
--		tbl.icon = icon; tbl.duration = max(mhDuration,ohDuration);
--		tbl.endTime = min(mhEndTime,ohEndTime);
--	end
end

LWE.RegisterCallback(addon, addon.UpdateWeaponEnchants);

local WeaponEnchantToolTip = CreateFrame("GameTooltip", addon:GetName().."WeaponEnchantToolTip", WorldFrame, "GameTooltipTemplate");
WeaponEnchantToolTip:SetScript("OnLoad", function() this:SetOwner(WorldFrame, "ANCHOR_NONE"); end)

local GetWeaponEnchant = function(itemId)
	WeaponEnchantToolTip:ClearAllPoints()
	WeaponEnchantToolTip:SetOwner(WorldFrame, "ANCHOR_NONE")
	WeaponEnchantToolTip:ClearLines()
	hasItem, _, _ = WeaponEnchantToolTip:SetInventoryItem("player", itemId)
	if (hasItem) then
		for iToolTipLine = 1, WeaponEnchantToolTip:NumLines(), 1 do
			local ttText = _G[WeaponEnchantToolTip:GetName().."TextLeft"..iToolTipLine]:GetText() or ""
			local _, _, enchantName, enchantRank, enchantDuration = string.find(ttText, "^([%a]+) (%d+) %((%d+) min%).*$");

			if (enchantName) then
				return enchantName;
			end
		end
	end

	return;
end

local GetDetailedWeaponEnchantInfo = function()
	local _, mainHandExpiration, _, _, offHandExpiration = GetWeaponEnchantInfo();

	local mainHandEnchantName	= GetWeaponEnchant(16);
	local offHandEnchantName	= GetWeaponEnchant(17);
	
	mainHandExpiration	= mainHandEnchantName and mainHandExpiration / 1000 or 0;
	offHandExpiration	= offHandEnchantName and offHandExpiration / 1000 or 0;

	return mainHandEnchantName, mainHandExpiration, offHandEnchantName, offHandExpiration;
end

function addon:TESTING()
	local mainHandEnchantName, mainHandExpiration, offHandEnchantName, offHandExpiration = GetDetailedWeaponEnchantInfo()

	if (mainHandEnchantName) then sCore:Print("MH: "..mainHandEnchantName.." ("..mainHandExpiration..")");
	else sCore:Print("MH: NONE"); end

	if (offHandEnchantName) then sCore:Print("OH: "..offHandEnchantName.." ("..offHandExpiration..")");
	else sCore:Print("OH: NONE"); end
end

local CheckWeaponEnchants = function()
	addon:TESTING();
end

-- Event Handlers
addon:SetScript("OnEvent", function(self, event ,...)
	if (event == "PLAYER_LOGIN") then
		addon:SetParent(UIParent);
		addon:ClearAllPoints();
		addon:SetPoint("CENTER");
		addon:SetWidth(buttonSize * 5 + buttonPadding * 4);
		addon:SetHeight(buttonSize * 2 + buttonPadding);
		addon:SetFrameStrata("LOW");
		
		if (not ALWAYS_VISIBLE) then
			addon:SetAlpha(0);
		end

		-- Create Maelstrom Tick Frames
		addon.MaelstromTicks = {};
		for i = 1, 5 do
			local tickButton = addon:CreateButton(addon:GetName().."MaelstromTick"..i, tickBorderColors[0], [[Interface\Addons\sEnhancer\media\overlay]], tickColors[0]);
			tickButton:ClearAllPoints();
			tickButton.colorTable = "TICK";
			if (i == 1) then
				tickButton:SetPoint("TOPLEFT");
			else
				tickButton:SetPoint("LEFT", addon.MaelstromTicks[i - 1], "RIGHT", buttonPadding, 0);
			end
			addon.MaelstromTicks[i] = tickButton;
		end
		
		-- Init Maelstrom Tick Count
		addon:SetMaelstromTicks(0);
		
		-- Create (De)Buff Frames
		addon.SpellButtons = {};
		for i = 1, 5 do
			local spellButton = addon:CreateButton(addon:GetName().."SpellButton"..i, spellBorderColors[0], addon.SpellData[addon.SpellButtonsOrder[i]][3], spellColors[0]);
			spellButton:ClearAllPoints();
			spellButton.colorTable = "SPELL";
			if (i == 1) then
				spellButton:SetPoint("TOPLEFT", addon.MaelstromTicks[1], "BOTTOMLEFT", 0, buttonPadding * -1);
			else
				spellButton:SetPoint("LEFT", addon.SpellButtons[i - 1], "RIGHT", buttonPadding, 0);
			end
			addon.SpellButtons[i] = spellButton;
		end

	elseif (event == "PLAYER_REGEN_ENABLED") then
		-- OOC: Hide Status Frames
		addon:ToggleVisibility(false);
	elseif (event == "PLAYER_REGEN_DISABLED") then
		-- IC: Display Status Frames
		addon:ToggleVisibility(true);
	elseif (event == "UNIT_AURA") then
		if (arg1 ~= nil and arg1 == "player") then
			-- Update Maelstrom Ticks
			addon:SetMaelstromTicks(GetMaelstromWeaponTicks())

			-- Weapon Enchants
--			local hasMainHandEnchant, mainHandExpiration, _, hasOffHandEnchant, offHandExpiration, _ = GetWeaponEnchantInfo()
--			sCore:Print(mainHandExpiration);
--			if (not hasMainHandEnchant or 
-- exptime - gettime() = sec
--			local expirationTime, index, count = UnitHasBuff("player", "Flametongue");
--			sCore:Print(expirationTime);
		end
	elseif (event == "COMBAT_LOG_EVENT_UNFILTERED") then
--		sCore:Print(arg2);
		if (arg6 == UnitGUID("player") and arg2 == "ENCHANT_REMOVED") then
--			sCore:Print("-wf/tf")
--			CheckWeaponEnchants();
--			sCore:Print("");
		elseif (arg6 == UnitGUID("player") and arg2 == "ENCHANT_APPLIED") then
--			sCore:Print("+wf/tf")
--			CheckWeaponEnchants();
--			sCore:Print("");
		end
	elseif (event == "UNIT_SPELLCAST_SUCCEEDED") then
		if (arg1 == "player") then
			if (arg2 == "Windfury Weapon" or arg2 == "Flametongue Weapon") then
--				sCore:Print("+wf/tf");
--				CheckWeaponEnchants();
--				sCore:Print("");
			end
		end
	end
end)

-- Done
_G["sEnhancer"] = addon

