--[[ 

	Enhancement Shaman PVE Helper
	Martin Karer / Sezz, 2010

	Changelog:
	2010-09-19:
		[*] Replaced Shamanstic Rage with Unleash Elements

	2010-09-16:
		[:] Cataclysm Update
			Please note: I have absolutely no idea about the priorities at L85...

	2010-07-03:
		[*] Replaced custom table.find with Blizzard's tContains

	2010-06-21:
		[+] Dual-Specc support

	2010-06-19:
		[*] Removed UNIT_AURA Event (-> COMBAT_LOG_EVENT_UNFILTERED)
		[*] Another Shamanistic Rage Cooldown Display Fix
		[*] Performance Optimizations

	2010-06-11:
		[*] Performance Optimizations

	2010-03-31:
		[*] Performance Optimizations

	2010-02-10:
		[+] Combined Weapon Enchant Frames
		[+] Cooldown Frames (Tested with OmniCC)
		[+] Shamanistic Rage + Cooldown (CD doesn't always work?)
		[+] Fire Nova Cooldown

	2010-01-30:
		[*] Fixed fire totem check for low level Shamans

	2010-01-20:
		[+] Fire totem check
		[+] Fire nova check

	2010-01-19:
		[*] Removed nearly all includes textures
		[+] Flame Shock Debuff Check
		
	2010-01-18:
		[+] Re-Design
		[+] Maelstrom Weapon Tick counter
		[+] Spell Icons
		[+] Weapon Imbue Checks 
		
	2010-01-17:
		[~] Testing: Maelstrom Weapon tick counter

--]]

if (select(2, UnitClass("player")) ~= "SHAMAN") then
	return false;
end

local addon = CreateFrame("Frame", "sEnhancer");
local _G = getfenv(0);
local LWE = LibWeaponEnchant;
local isEnhancer = false;

local buttonSize = 30;
local buttonBorderSize = 2;
local buttonPadding = 8;

local FADE_TIME = 0.1;
local ALWAYS_VISIBLE = false; -- ONLY FOR DEBUGGING, MOST EVENTS ARE DISABLED OOC!
local BUFF_WARN_TIME = 7; -- FS only ticks every 3 seconds!
local GLOBAL_COOLDOWN = 2.0;

-- Local Terror
local GetSpellCooldown = GetSpellCooldown;
local UnitAffectingCombat = UnitAffectingCombat;
local UnitAura = UnitAura;
local GetTotemInfo = GetTotemInfo;
local FIRE_TOTEM_SLOT = FIRE_TOTEM_SLOT;
local UnitGUID = UnitGUID;

-- Colors/Alpha Values
local tickGlossAlpha = {
	[0] = 0.25,
	[1] = 0.50,
	[2] = 0.50,
	}

local tickColors = {
	[0] = { ["r"] = 0, ["g"] = 0, ["b"] = 0, ["a"] = 1 }, -- 0 ticks
	[1] = { ["r"] = 0/255, ["g"] = 158/255, ["b"] = 197/255, ["a"] = 1 }, -- 1-4 ticks
	[2] = { ["r"] = 204/255, ["g"] = 0/255, ["b"] = 255/255, ["a"] = 1 }, -- 5 ticks
	}

local tickBorderColors = {
	[0] = { ["r"] = .6, ["g"] = .6, ["b"] = .6, ["a"] = 1 },
	[1] = { ["r"] = 1, ["g"] = 1, ["b"] = 1, ["a"] = 1 },
	[2] = { ["r"] = 1, ["g"] = 1, ["b"] = 1, ["a"] = 1 },
	}

local spellColors = {
	[0] = { ["r"] = 1, ["g"] = 1, ["b"] = 1, ["a"] = .2 }, -- fadet out
	[1] = { ["r"] = 0.3, ["g"] = 0.5, ["b"] = 1, ["a"] = 1 }, -- needs attention soon
	[2] = { ["r"] = 1, ["g"] = 1, ["b"] = 1, ["a"] = 1 }, -- warning
	}

local spellBorderColors = {
	[0] = { ["r"] = 1, ["g"] = 1, ["b"] = 1, ["a"] = 0.5 },
	[1] = { ["r"] = 1, ["g"] = 0.5, ["b"] = 0, ["a"] = 1.0 },
	[2] = { ["r"] = 1, ["g"] = 0, ["b"] = 0, ["a"] = 1.0 },
	}

-- Spell Data
addon.SpellButtonsOrder = { "Flame Shock", "Searing Totem", "Fire Nova", "Windfury Weapon", "Unleash Elements" }
addon.SpellData = {
	["Flame Shock"]			= { GetSpellInfo(8050) },
	["Searing Totem"]		= { GetSpellInfo(3599) },
	["Fire Nova"]			= { GetSpellInfo(1535)  },
	["Windfury Weapon"]		= { GetSpellInfo(8232) },
	["Flametongue Weapon"]	= { GetSpellInfo(8024) },
	["Unleash Elements"]	= { GetSpellInfo(73680) },
	};

addon.SpellButtonsIndex = {}
for i = 1, 5 do
	addon.SpellButtonsIndex[addon.SpellButtonsOrder[i]] = i;
end

-- In Combat Events
local _eventsCombat = {
	"PLAYER_TARGET_CHANGED", -- Flame Shock
	"COMBAT_LOG_EVENT_UNFILTERED", -- Maelstrom Weapon, Flame Shock
	"PLAYER_TOTEM_UPDATE", -- Fire Totem, Fire Nova
	"SPELL_UPDATE_COOLDOWN", -- Cooldowns (Fire Nova, Unleash Elements)
	}

-- Cooldowns
local cdElapsed = 0
local cdTimer = 0
local cdWatched = {};
local cdOnUpdate = function(_, update)
    cdElapsed = cdElapsed + update
    if (cdElapsed > 0.10) then
		for i, spellName in ipairs(cdWatched) do
			local cdStart, cdDuration = GetSpellCooldown(spellName)
			if (not (cdStart > 0 and cdDuration > 0)) then
				-- Spell Ready!
				addon:CheckCooldown(spellName);
				cdWatched[i] = nil;
			end
		end

    	if (#cdWatched == 0) then
    		-- Disable Cooldown Watching
    		addon:SetScript("OnUpdate", nil);
    	end
    end
end

function addon:WatchCooldown(spellName)
	-- Watch Cooldown
	local cdStart, cdDuration, cdEnabled = GetSpellCooldown(spellName);
	if (cdStart == nil) then
		-- Unknown Spell
		return true;
	end
	
	if (cdStart > 0 and cdDuration > GLOBAL_COOLDOWN and not (tContains(cdWatched, spellName))) then
		-- Add Spell
		table.insert(cdWatched, spellName);
		-- Enable Cooldown Frame
		CooldownFrame_SetTimer(addon.SpellButtons[addon.SpellButtonsIndex[spellName]].CooldownFrame, cdStart, cdDuration, cdEnabled);
		-- Enable Update Script
		if (#cdWatched == 1) then
			addon:SetScript("OnUpdate", cdOnUpdate);
		end
	end
	
	-- Returns TRUE if spell is on cooldown or FALSE if spell is ready...
	return (cdStart > 0 and cdDuration > GLOBAL_COOLDOWN) or false;
end

function addon:CheckCooldown(spellName)
	if (UnitAffectingCombat("player")) then
		local cdCheckFunc = true;
		if (addon.SpellDataCooldownCheck[spellName] ~= nil) then
			cdCheckFunc = addon.SpellDataCooldownCheck[spellName]();
		end
		
		addon:ToggleSpellStatus(addon.SpellButtonsIndex[spellName], cdCheckFunc and (addon:WatchCooldown(spellName) and 0 or 2) or 0);
	else
		addon:ToggleSpellStatus(addon.SpellButtonsIndex[spellName], 0);
	end
end

-- Flame Shock
function addon:CheckFlameShockDebuff(forceDebuffState)
	local debuffState = 0;
	if (forceDebuffState ~= nil) then
		debuffState = forceDebuffState;
	else
		if (UnitExists("target") and UnitCanAttack("player", "target") and not UnitIsDead("target")) then
		
			local _, _, _, _, _, _, expirationTime, unitCaster = UnitAura("target", "Flame Shock", nil, "HARMFUL");
			if (expirationTime and unitCaster == "player") then
				debuffState = expirationTime - GetTime() < BUFF_WARN_TIME and 1 or 0;
			else
				debuffState = 2;
			end
		end
	end

	addon:ToggleSpellStatus(addon.SpellButtonsIndex["Flame Shock"], debuffState);
end

-- Fire Totem, Fire Nova
local IsFireTotemActive = function()
	local _, totemName = GetTotemInfo(FIRE_TOTEM_SLOT);
	return totemName ~= "";
end

function addon:CheckFireTotem()
	if (UnitAffectingCombat("player")) then
		addon:ToggleSpellStatus(addon.SpellButtonsIndex["Searing Totem"], IsFireTotemActive() and 0 or 2);
	else
		addon:ToggleSpellStatus(addon.SpellButtonsIndex["Searing Totem"], 0);
	end
end

-- Maelstrom Weapon
local GetMaelstromWeaponTicks = function()
	local _, _, _, ticks = UnitAura("player", "Maelstrom Weapon", nil, "HELPFUL");
	return ticks or 0;
end

-- Color Fading
local getFadeStep = function(oldNum, newNum)
	local numDiff = oldNum - newNum;
	return (numDiff ~= 0 and numDiff / 100 or 0);
end

local calculcateFadeColor = function(oldColors, fadeColors, fadePercent) 
	return oldColors["r"] - (fadePercent * getFadeStep(oldColors["r"], fadeColors["r"])),
		   oldColors["g"] - (fadePercent * getFadeStep(oldColors["g"], fadeColors["g"])),
		   oldColors["b"] - (fadePercent * getFadeStep(oldColors["b"], fadeColors["b"])),
		   oldColors["a"] - (fadePercent * getFadeStep(oldColors["a"], fadeColors["a"]));
end

local tickButton_OnUpdateTimer = 0
local tickButton_OnUpdate = function(tickButton, elapsed)
	tickButton_OnUpdateTimer = tickButton_OnUpdateTimer + (elapsed or 0);
	if (tickButton.fadeInStartTime ~= nil and tickButton.tickStateOld ~= tickButton.tickState) then
		-- Fade...
		local fadePercent = ((GetTime() - tickButton.fadeInStartTime) / FADE_TIME) * 100;
		if (fadePercent > 100) then fadePercent = 100; end;
		
		if (tickButton_OnUpdateTimer > 0.1 or fadePercent == 100) then
			if (tickButton.colorTable == "TICK") then
				tickButton.Icon:SetVertexColor(calculcateFadeColor(tickColors[tickButton.tickStateOld], tickColors[tickButton.tickState], fadePercent));
				tickButton.Border:SetVertexColor(calculcateFadeColor(tickBorderColors[tickButton.tickStateOld], tickBorderColors[tickButton.tickState], fadePercent));
			elseif (tickButton.colorTable == "SPELL") then
				tickButton.Icon:SetVertexColor(calculcateFadeColor(spellColors[tickButton.tickStateOld], spellColors[tickButton.tickState], fadePercent));
				tickButton.Border:SetVertexColor(calculcateFadeColor(spellBorderColors[tickButton.tickStateOld], spellBorderColors[tickButton.tickState], fadePercent));
			end

			tickButton.Gloss:SetAlpha(tickGlossAlpha[tickButton.tickStateOld] - (fadePercent * getFadeStep(tickGlossAlpha[tickButton.tickStateOld], tickGlossAlpha[tickButton.tickState])));

			if (fadePercent == 100) then
				tickButton.fadeInStartTime = nil;
				tickButton:SetScript("OnUpdate", nil);
			end
		end
	else
		tickButton.fadeInStartTime = nil;
		tickButton:SetScript("OnUpdate", nil);
	end
end

function addon:SetMaelstromTicks(tickNum)
	for i = 1, 5 do
		local tickButton = addon.MaelstromTicks[i];
		if (tickButton) then
			local tickState	= ((tickNum == 5) and 2 or ((i <= tickNum) and 1 or 0));
			if (tickState ~= tickButton.tickState) then
				tickButton.tickStateOld = tickButton.tickState;
				tickButton.tickState = tickState;
				tickButton.fadeInStartTime = GetTime();
				tickButton:SetScript("OnUpdate", tickButton_OnUpdate);
			end
		else
			return;
		end
	end
end

-- IC/OOC Visibility
function addon:ToggleVisibility(showAddon)
	if (showAddon and addon:GetAlpha() ~= 1) then
		UIFrameFadeIn(sEnhancer, 0.2);
	elseif (not showAddon and addon:GetAlpha() ~= 0) then
		UIFrameFadeOut(sEnhancer, 0.2);
	end
end

-- Button Template
-- TODO: XML
function addon:CreateButton(buttonName, borderColor, iconTexture, iconColor)
	local spellButton = CreateFrame("Button", buttonName, addon);
	spellButton:SetWidth(buttonSize);
	spellButton:SetHeight(buttonSize);
	spellButton:SetParent(addon);
	spellButton:EnableMouse(false);
	spellButton.tickState = 0;

	-- Border
	local spellButtonBorder = spellButton:CreateTexture(buttonName.."Border", "BACKGROUND");
	spellButtonBorder:SetBlendMode("BLEND");
	spellButtonBorder:SetAllPoints(spellButton);
	spellButtonBorder:SetTexture([[Interface\Tooltips\UI-Tooltip-Background]]);
	spellButtonBorder:SetTexCoord(0.08, 0.92, 0.08, 0.92);
	spellButtonBorder:SetVertexColor(1, 1, 1, 0.5);
	spellButton.Border = spellButtonBorder;

	-- Black Icon BG
	local spellButtonIconBackground = spellButton:CreateTexture(buttonName.."IconBackground", "BORDER");
	spellButtonIconBackground:SetBlendMode("DISABLE");
	spellButtonIconBackground:ClearAllPoints();
	spellButtonIconBackground:SetPoint("CENTER");
	spellButtonIconBackground:SetWidth(buttonSize - 2 * buttonBorderSize);
	spellButtonIconBackground:SetHeight(buttonSize - 2 * buttonBorderSize);
	spellButtonIconBackground:SetTexture([[Interface\Addons\sEnhancer\media\overlay]]);
	spellButtonIconBackground:SetTexCoord(0.08, 0.92, 0.08, 0.92);
	spellButtonIconBackground:SetVertexColor(0, 0, 0, 1);

	-- Icon
	local spellButtonIcon = spellButton:CreateTexture(buttonName.."Icon", "ARTWORK");
	spellButtonIcon:ClearAllPoints();
	spellButtonIcon:SetPoint("CENTER");
	spellButtonIcon:SetWidth(buttonSize - 2 * buttonBorderSize);
	spellButtonIcon:SetHeight(buttonSize - 2 * buttonBorderSize);
	spellButtonIcon:SetTexture(iconTexture);
	spellButtonIcon:SetTexCoord(0.08, 0.92, 0.08, 0.92);
	spellButtonIcon:SetVertexColor(iconColor["r"], iconColor["g"], iconColor["b"], iconColor["a"]);
	spellButton.Icon = spellButtonIcon;

	-- Gloss
	local spellButtonGloss = spellButton:CreateTexture(buttonName.."Gloss", "OVERLAY");
	spellButtonGloss:SetBlendMode("ADD");
	spellButtonGloss:ClearAllPoints();
	spellButtonGloss:SetPoint("CENTER");
	spellButtonGloss:SetWidth(buttonSize - 2 * buttonBorderSize);
	spellButtonGloss:SetHeight(buttonSize - 2 * buttonBorderSize);
	spellButtonGloss:SetTexture([[Interface\Addons\sEnhancer\media\gloss]]);
	spellButtonGloss:SetVertexColor(1,1,1,1);
	spellButtonGloss:SetAlpha(0.25);
	spellButton.Gloss = spellButtonGloss;
	
	-- Cooldown
	local spellButtonCooldown = CreateFrame("Cooldown", nil, spellButton, "CooldownFrameTemplate")
	spellButtonCooldown:ClearAllPoints();
	spellButtonCooldown:SetPoint("CENTER");
	spellButtonCooldown:SetWidth(buttonSize - 2 * buttonBorderSize);
	spellButtonCooldown:SetHeight(buttonSize - 2 * buttonBorderSize);
	spellButton.CooldownFrame = spellButtonCooldown;

	return spellButton;
end

function addon:ToggleSpellStatus(spellNum, newState)
	local spellButton = addon.SpellButtons[spellNum];
	
	if (spellButton and spellButton.tickState ~= newState) then
		spellButton.tickStateOld = spellButton.tickState;
		spellButton.tickState = newState;
		spellButton.fadeInStartTime = GetTime();
		spellButton:SetScript("OnUpdate", tickButton_OnUpdate);
	end
end

-- Weapon Enchants
-- Using LibWeaponEnchant by Aezay
-- http://www.wowinterface.com/downloads/info8437.html
addon.UpdateWeaponEnchants = function(mhSlot, mhEnchant, mhDuration, mhEndTime, ohSlot, ohEnchant, ohDuration, ohEndTime)
	addon:CheckWeaponEnchants();
end

function addon:CheckWeaponEnchants()
	local mhSlot, mhEnchant, mhDuration, mhEndTime, ohSlot, ohEnchant, ohDuration, ohEndTime = LWE.GetEnchantData();
	local mhEnchantName = mhEnchant and mhEnchant:gsub(" %d+","") or "UNKNOWN";
	local ohEnchantName = ohEnchant and ohEnchant:gsub(" %d+","") or "UNKNOWN";
	addon:ToggleSpellStatus(addon.SpellButtonsIndex["Windfury Weapon"], mhEnchantName == "Windfury" and ohEnchantName == "Flametongue" and 0 or 2);
end

LWE.RegisterCallback(addon, addon.UpdateWeaponEnchants);

-- Additional checks for cooldown watching
-- CD Ready && SpellDataCooldownCheck = Highlight Button
addon.SpellDataCooldownCheck = {
	["Fire Nova"] = IsFireTotemActive,
	};

-- Event Handlers
addon:RegisterEvent("PLAYER_LOGIN");
addon:RegisterEvent("PLAYER_REGEN_ENABLED");
addon:RegisterEvent("PLAYER_REGEN_DISABLED");
addon:RegisterEvent("ACTIVE_TALENT_GROUP_CHANGED");

function addon:ACTIVE_TALENT_GROUP_CHANGED()
	-- Check Primary Specc
	local primaryTalentTab = 0;
	local pointsSpentMaximum = 0;
	for i = 1, GetNumTalentTabs() do
		local _, _, _, _, pointsSpent = GetTalentTabInfo(i);
		if (pointsSpent > pointsSpentMaximum) then
			pointsSpentMaximum = pointsSpent;
			primaryTalentTab = i;
		end
	end
	
	isEnhancer = (primaryTalentTab == 2);
end

addon:RegisterEvent("PLAYER_ALIVE");
function addon:PLAYER_ALIVE(event)
	addon:ACTIVE_TALENT_GROUP_CHANGED();
	addon:UnregisterEvent(event);
end

addon:RegisterEvent("PLAYER_ENTERING_WORLD");
function addon:PLAYER_ENTERING_WORLD(event)
	addon:ACTIVE_TALENT_GROUP_CHANGED();
	addon:UnregisterEvent(event);
end

function addon:PLAYER_LOGIN()
	addon:SetScale(0.77);
	addon:SetParent(UIParent);
	addon:ClearAllPoints();
	addon:SetPoint("CENTER", 0, -160);
	addon:SetWidth(buttonSize * 5 + buttonPadding * 4);
	addon:SetHeight(buttonSize * 2 + buttonPadding);
	addon:SetFrameStrata("LOW");

	if (not ALWAYS_VISIBLE) then
		addon:SetAlpha(0);
	end

	-- Create Maelstrom Tick Frames
	addon.MaelstromTicks = {};
	for i = 1, 5 do
		local tickButton = addon:CreateButton(addon:GetName().."MaelstromTick"..i, tickBorderColors[0], [[Interface\Addons\sEnhancer\media\overlay]], tickColors[0]);
		tickButton:ClearAllPoints();
		tickButton.colorTable = "TICK";
		if (i == 1) then
			tickButton:SetPoint("TOPLEFT");
		else
			tickButton:SetPoint("LEFT", addon.MaelstromTicks[i - 1], "RIGHT", buttonPadding, 0);
		end
		addon.MaelstromTicks[i] = tickButton;
	end

	-- Init Maelstrom Tick Count
	addon:SetMaelstromTicks(0);

	-- Create (De)Buff Frames
	addon.SpellButtons = {};
	for i = 1, 5 do
		local spellButton = addon:CreateButton(addon:GetName().."SpellButton"..i, spellBorderColors[0], addon.SpellData[addon.SpellButtonsOrder[i]][3], spellColors[0]);
		spellButton:ClearAllPoints();
		spellButton.colorTable = "SPELL";
		if (i == 1) then
			spellButton:SetPoint("TOPLEFT", addon.MaelstromTicks[1], "BOTTOMLEFT", 0, buttonPadding * -1);
		else
			spellButton:SetPoint("LEFT", addon.SpellButtons[i - 1], "RIGHT", buttonPadding, 0);
		end
		addon.SpellButtons[i] = spellButton;
	end

	-- Check Weapon Enchants
	addon:CheckWeaponEnchants();

	-- Done
	addon:UnregisterEvent("PLAYER_LOGIN");
end

function addon:PLAYER_REGEN_ENABLED()
	-- Out Of Combat: DISABLE
	if (not isEnhancer) then return; end

	-- Hide Status Frames
	if (not ALWAYS_VISIBLE) then
		addon:ToggleVisibility(false);
	end

	-- Unregister Events
	for _, v in pairs(_eventsCombat) do
		addon:UnregisterEvent(v);
	end

	-- Fade-Out Cooldowns/Totems
	addon:CheckFireTotem();
	addon:CheckFlameShockDebuff();
	addon:CheckCooldown("Fire Nova");
	addon:CheckCooldown("Unleash Elements");

	-- Disable Cooldown Checks
	addon:SetScript("OnUpdate", nil);
	cdWatched = {}
end

function addon:PLAYER_REGEN_DISABLED()
	-- In Combat: ENABLE
	if (not isEnhancer) then return; end

	-- Register Events
	for _, v in pairs(_eventsCombat) do
		addon:RegisterEvent(v);
	end

	-- Initial Checks
	addon:CheckFireTotem();
	addon:CheckFlameShockDebuff();
	addon:CheckCooldown("Fire Nova");
	addon:CheckCooldown("Unleash Elements");

	-- Display Status Frames
	if (not ALWAYS_VISIBLE) then
		addon:SetMaelstromTicks(GetMaelstromWeaponTicks());
		addon:ToggleVisibility(true);
	end
end

function addon:PLAYER_TARGET_CHANGED()
	-- Check Flame Shock Debuff
	addon:CheckFlameShockDebuff();
end

function addon:COMBAT_LOG_EVENT_UNFILTERED(self, ...)
	local _, arg2, arg3, _, _, arg6, _, _, _, arg10, _, _, arg13 = ...;

	-- Flame Shock Periodic Damage
	if (arg3 == UnitGUID("player")) then
		-- Player > Target
		if (arg6 == UnitGUID("target") and arg10 == "Flame Shock") then
			if (arg2 == "SPELL_AURA_APPLIED" or arg2 == "SPELL_AURA_REFRESH") then
				addon:CheckFlameShockDebuff(0);
			elseif (arg2 == "SPELL_AURA_REMOVED") then
				addon:CheckFlameShockDebuff(2);
			elseif (arg2 == "SPELL_PERIODIC_DAMAGE") then
				-- Flame Shock Applied/Refreshed/Removed
				addon:CheckFlameShockDebuff();
			end
		else
			if (arg6 == arg3 and arg2 ~= "SPELL_AURA_REFRESH") then
				-- Source = Target = Player
				if (arg10 == "Maelstrom Weapon") then
					-- Maelstrom Weapon
					if (arg2 == "SPELL_AURA_APPLIED_DOSE") then
						addon:SetMaelstromTicks(arg13);
					elseif (arg2 == "SPELL_AURA_APPLIED") then
						addon:SetMaelstromTicks(1);
					elseif (arg2 == "SPELL_AURA_REMOVED") then
						addon:SetMaelstromTicks(0);
					end
				elseif (arg10 == "Unleash Elements") then
					-- Unleash Elements
					addon:CheckCooldown("Unleash Elements");
				end
			end
		end
	end
end


function addon:PLAYER_TOTEM_UPDATE(self, totemSlot)
	if (totemSlot == FIRE_TOTEM_SLOT) then
		addon:CheckFireTotem();
		addon:CheckCooldown("Fire Nova");
	end
end

function addon:SPELL_UPDATE_COOLDOWN()
	addon:CheckFireTotem();
	addon:CheckCooldown("Fire Nova");
	addon:CheckCooldown("Unleash Elements");
end

addon:SetScript("OnEvent",function(self, event, ...) self[event](self, event, ...); end);

-- Done
_G["sEnhancer"] = addon;

